/*
 * The declaration for the probes inserted in the Pervasive Postgres Source Code
 *
 * Portions copyright Pervasive Software, Inc. 2005. All rights reserved.
 */

provider Postgres
{
	/* Category : Query */

	/*
	 * cost__of__query : Fired after the cost of the query is calculated
	 *
	 * Arguments
	 *		cost : cost of the query being executed
	 * NOTE:
	 *		cost is float. Since dtrace does not support float, it is being passed as char*
	 *	It can be converted back to float using atof() or atod().
	 *
	 */
	probe cost__of__query(char* cost);		

	/*
	 * execution__start : Fired when the execution of a query starts
	 *
	 * Arguments
	 *		query : SQL query being executed
	 *
	 */
	probe execution__start(char* query);

	/*
	 * execution__finish : Fired when the execution of the query is finished
	 *
	 * Arguments
	 *		query : SQL query that finished executing
	 *
	 */
   	probe execution__finish(char* query);


	/* Category : Cache */

	/*
	 * cache__hit : Fired when a cache hit occurs while executing a query
	 */
	probe cache__hit(void);

	/*
	 * cache__miss : Fired when a cache miss occurs while executing a query
	 */
	probe cache__miss(void);

	
	/* Category : Triggers  */

	/*
	 * trigger__fire : Fired when a trigger gets fired
	 *
	 * Arguments
	 *		eventTime  : When the trigger is to be executed (BEFORE or AFTER)
	 *		operation  : What causes the trigger to fire (INSERT, DELETE, UPDATE)
	 *		eventFor   : Whether the trigger is fired for ROW or STATEMENT  
	 * 		eventOn	   : Relation name on which the trigger is created
	 *
	 */
	probe trigger__fire(char *eventTime, char *operation, char *eventFor, char *eventOn);

	
	/* Category : Transaction */

	/*
	 * transaction__start : Fired when a transaction is started
	 *
	 * Arguments
	 *		id	: Transaction ID
	 *
	 */
	probe transaction__start(int id);
	
	/*
	 * transaction__commit : Fired when a transaction is committed
	 *
	 * Arguments
	 *		id	: Transaction ID
	 *
	 */
	probe transaction__commit(int id);
	
	/*
	 * transaction__rollback : Fired when a transaction is rolled back
	 *
	 * Arguments
	 *		id	: Transaction ID
	 *
	 */
	probe transaction__rollback(int id);


	/* Category : Locks */

	/*
	 * lock__acquire : Fired when a lock is acquired
	 *
	 * Arguments
	 *		xid   : Transaction Id
	 *		lmid  : Lock method Id (User lock or Default)
	 *		owner : Owner of the lock (If available)
	 *		dbId  : Database Id 
	 *
	 */
	probe lock__acquire(int xid, int lmid, char *owner, int dbId);

	/*
	 * lock__release : Fired when a lock is released
	 *
	 * Arguments
	 *		xid   : Transaction Id
	 *		lmid  : Lock method Id (Invalid or Default or User lock)
	 *		owner : Owner of the lock (If available)
	 *		dbId  : Database Id 
	 *
	 */
	probe lock__release(int xid, int lmid, char *owner, int dbId);

	/*
	 * lock__release__all : Fired when all locks are released
	 *
	 * Arguments
	 *		lmid : Lock method Id
	 *
	 * NOTE:
	 *		All locks of type lmid are released across transactions and connections.
	 *
	 */
	probe lock__release__all(int lmid);

	/* Category : User */
	
	/*
	 * login__succeed : Fired when a user logged into the database successfully
	 *
	 * Arguments
	 *		name : user name
	 *
	 */
	probe login__succeed(char* name);

	/*
	 * login__fail : Fired when a user failed to login to the database successfully
	 *
	 * Arguments
	 *		name : user name
	 *
	 */
	probe login__fail(char* name);

	/* 
	 * access__check : Fired when a user tries to access an object without necessary permissions
	 *
	 * Arguments
	 *		kind : The type of object that is being accessed. (Relation, Database, Type, etc) 	
	 *		name : Name of the object that is being accessed
	 *
	 */
	probe access__check(char *kind, char *name);


	/* Category : Deadlock */

	/*
	 * deadlock__detected : Fired when a deadlock is detected
	 */
	probe deadlock__detected(void); 


	/* Category : Vacuum */

	/*
	 * vacuum__start : Fired when vacuum starts on the database
	 */
	probe vacuum__start(void);

	/*
	 * vacuum__stop : Fired when the vacuum stops on the database
	 */
	probe vacuum__stop(void);


	/* Category : Database Metadata */

	/*
	 * call__create : Fired when an object is created in a database
	 *
	 * Arguments
	 *		type 		: Type of object being created (TABLE, STATEMENT, TABLESPACE)
	 *		name		: Name of the object created
	 *
	 *	If the object created is SCHEMA or TABLESPACE
	 *		arg1		: The user name who owns the SCHEMA or TABLESPACE
	 *		arg2		: NULL
	 *
	 *	If the object created is STATEMENT
	 *		arg1		: The schema name in which the STATEMENT is created
	 *		arg2		: Contains the catalog name of the STATEMENT
	 *		
	 */
	probe call__create(char *type, char *name, char *arg1, char *arg2);

	/*
	 * call__alter : Fired when an object is altered in a database
	 *
	 * Arguments
	 *		type 		: Type of object being altered
	 *
	 *	If the object is TABLE or OWNER
	 *		name		: Relation name
	 *		schema		: Schema to which the TABLE belongs to
	 *		catalog		: Catalog to which the TABLE belongs to
	 *
	 *	If the object is DOMAIN
	 *		name		: The name of the "column or constraint" in case of a DOMAIN
	 *		schema		: NULL
	 * 		catalog		: NULL
	 *
	 */
	probe call__alter(char *type, char *name, char *schema, char *catalog);

	/*
	 * call__drop : Fired when an object is dropped from a database
	 *
	 * Arguments
	 *		type 		: Type of object being dropped (TABLE, SEQUENCE, TYPE, etc)
	 *		relname 	: Relation Name
	 *		schemaname  	: Schema name
	 * 		catalogname 	: Catalog or database Name
	 *
	 */
	probe call__drop(char *type, char *relname, char *schemaname, char *catalogname);

};
